import { get as get$1 } from 'http';
import { get } from 'https';

// src/scheme.ts
function regex_escape(str) {
  return str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
}
function make_regex(schemes) {
  return new RegExp("^(" + schemes.map(regex_escape).join("|") + "):");
}
async function fetch(url, agent, cache, onfetch = function noop() {
}) {
  let contents = cache.get(url);
  if (contents)
    return contents;
  let options = agent ? { agent } : {};
  contents = await new Promise((resolve, reject) => {
    function fetch_recur() {
      onfetch(url);
      let get$2 = url.startsWith("https") ? get : get$1;
      let req = get$2(url, options, (res) => {
        if ([301, 302, 307].includes(res.statusCode)) {
          url = new URL(res.headers.location, url).toString();
          fetch_recur();
          req.destroy();
        } else if (res.statusCode === 200) {
          let chunks = [];
          res.on("data", (chunk) => chunks.push(chunk));
          res.on("end", () => resolve(Buffer.concat(chunks)));
        } else {
          reject(new Error(`GET ${url} failed: status ${res.statusCode}`));
        }
      }).on("error", reject);
    }
    fetch_recur();
  });
  cache.set(url, contents);
  return contents;
}

// src/plugin.ts
function http({
  filter,
  agent = null,
  schemes = {},
  cache = /* @__PURE__ */ new Map(),
  onfetch
} = {}) {
  return {
    name: "http",
    setup({ onResolve, onLoad }) {
      let scheme_filter;
      let scheme_keys = Object.keys(schemes);
      if (scheme_keys.length > 0) {
        scheme_filter = make_regex(scheme_keys);
      }
      let filter_func = filter === void 0 ? function(url) {
        return true;
      } : typeof filter === "function" ? filter : function(url) {
        return filter.test(url);
      };
      let ext_to_loader = {
        ".js": "js",
        ".mjs": "js",
        ".cjs": "js",
        ".jsx": "jsx",
        ".ts": "ts",
        ".cts": "ts",
        ".mts": "ts",
        ".tsx": "tsx",
        ".css": "css",
        ".json": "json",
        ".txt": "text"
      };
      if (scheme_filter) {
        onResolve({ filter: scheme_filter }, ({ path }) => {
          let key = scheme_keys.find((prefix2) => path.startsWith(prefix2 + ":"));
          let prefix = schemes[key];
          let search = "";
          let idx = prefix.indexOf("?");
          if (idx !== -1) {
            search = prefix.slice(idx);
            prefix = prefix.slice(0, idx);
          }
          let url = prefix + path.slice(key.length + 1) + search;
          if (filter_func(url)) {
            return { path: url, namespace: "http-url" };
          }
        });
      }
      onResolve({ filter: /^https?:\/\// }, (args) => {
        if (filter_func(args.path)) {
          return { path: args.path, namespace: "http-url" };
        }
      });
      onResolve({ filter: /.*/, namespace: "http-url" }, (args) => ({
        path: new URL(args.path, args.importer).toString(),
        namespace: "http-url"
      }));
      onLoad({ filter: /.*/, namespace: "http-url" }, async (args) => {
        let loader = "js";
        let idx = args.path.lastIndexOf(".");
        if (idx !== -1) {
          let ext = args.path.slice(idx);
          loader = ext_to_loader[ext] || loader;
        }
        let contents = await fetch(args.path, agent, cache, onfetch);
        return { contents, loader };
      });
    }
  };
}
var default_schemes = {
  unpkg: "https://unpkg.com/?module",
  jsdelivr: "https://esm.run/",
  esm: "https://esm.sh/?bundle"
};

export { http as default, default_schemes, http };
//# sourceMappingURL=index.mjs.map